.service-form-wrapper {
    padding: 20px 0;
}

.service-form-content {
    padding: 20px;
    margin: 30px auto;
    border: 1px solid #ddd;
    border-radius: 5px;
    max-width: 600px;
    box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
}

h2 {
    border-bottom: 1px solid #e0e0e0;
    padding-bottom: 15px;
    margin-bottom: 30px;
    font-weight: 600;
}

.form-control {
    width: 100%;
    padding: 5px;
    border: 1px solid #ccc;
    border-radius: 5px;
    transition: border 0.3s;
}

.form-control:focus {
    border-color: #007BFF;
    box-shadow: 0 0 5px rgba(0,123,255,0.2);
}

/* Design TextArea */
textarea.form-control {
    height:60px;
}