/* staff_form.css */
.staff-form-wrapper {
    margin: 15px auto;
    padding: 10px 0;
}

.staff-form-content {
    padding: 20px;
    border: 1px solid #ddd;
    border-radius: 5px;
    max-width: 1100px;
    margin: 0 auto;
    box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
}

h3 {
    border-bottom: 1px solid #e0e0e0;
    padding-bottom: 15px;
    margin-bottom: 20px;
    font-weight: 500;
}

.form-group {
    margin-bottom: 20px;
}

.form-group label {
    display: block;
    margin-bottom: 8px;
    font-weight: 500;
}

.form-control {
    width: 100%;
    padding: 2px 10px;
    border: 1px solid #ccc;
    border-radius: 5px;
    transition: border 0.3s;
}

.form-control:focus {
    border-color: #007BFF;
    box-shadow: 0 0 5px rgba(0,123,255,0.2);
}

.form-check {
    margin-bottom: 10px;
}

.form-check label {
    margin-left: 5px;
    font-weight: 400;
}

.btn {
    padding: 10px 20px;
    border: none;
    background-color: #007bff;
    color: #fff;
    border-radius: 5px;
    cursor: pointer;
    transition: background-color 0.3s;
}

.btn:hover {
    background-color: #0056b3;
}

/* ############################### */
.user-not-found {
    margin-top: 2px;
    margin-bottom: 20px;
    padding: 5px;
    background-color: #f9f9f9;
    border-radius: 4px;
    font-size: 0.9rem;
}

.user-not-found a {
    color: #007bff;
    text-decoration: underline;
}

.user-not-found small {
    color: #666;
}

small {
    color: #3d3d3d;
    font-size: 12px;
}

