{% load i18n %}
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{% translate 'Appointment Request Notification' %}</title>
    <style>
        body {
            font-family: 'Helvetica Neue', Helvetica, Arial, sans-serif;
            background-color: #f5f5f5;
            color: #333;
            padding: 20px;
            margin: 0;
        }

        .email-container {
            background-color: #ffffff;
            padding: 25px;
            margin: 0 auto;
            max-width: 650px;
            border-radius: 5px;
            box-shadow: 0 4px 8px rgba(0, 0, 0, 0.05);
        }

        h1 {
            color: #333;
            font-size: 24px;
        }

        p {
            font-size: 16px;
            line-height: 1.6;
        }

        .appointment-details {
            background-color: #f9f9f9;
            padding: 15px;
            margin-top: 20px;
            border-left: 5px solid #007bff;
        }

        .footer {
            margin-top: 30px;
            font-size: 14px;
            text-align: left;
            color: #999;
        }
    </style>
</head>
<body>
<div class="email-container">
    <h1>{% translate 'New Appointment Request' %}</h1>
    <p>{% translate 'Dear Admin,' %}</p>
    <p>{% translate 'You have received a new appointment request. Here are the details:' %}</p>

    <div class="appointment-details">
        <p><strong>{% translate 'Client Name' %}:</strong> {{ client_name }}</p>
        <p><strong>{% translate 'Service Requested' %}:</strong> {{ appointment.get_service_name }}</p>
        <p><strong>{% translate 'Appointment Date' %}:</strong> {{ appointment.appointment_request.date }}</p>
        <p><strong>{% translate 'Time' %}:</strong> {{ appointment.appointment_request.start_time }}
            - {{ appointment.appointment_request.end_time }}</p>
        <p><strong>{% translate 'Contact Details' %}:</strong> {{ appointment.phone }} | {{ appointment.client.email }}</p>
        <p><strong>{% translate 'Additional Info' %}:</strong> {{ appointment.additional_info|default:"N/A" }}</p>
    </div>

    <p>{% translate 'Please review the appointment request and take the necessary action.' %}</p>

    <div class="footer">
        <p>{% translate 'This is an automated message. Please do not reply directly to this email.' %}</p>
    </div>
</div>
</body>
</html>
