{% load i18n %}
<!-- Error Modal -->
<div class="modal fade" id="errorModal" tabindex="-1" aria-labelledby="errorModalLabel" aria-hidden="true">
    <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title" id="errorModalLabel">{% trans 'Error' %}</h5>
                <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"
                        onclick="closeErrorModal()"></button>
            </div>
            <div class="modal-body">
                <!-- Error message will be inserted here -->
                <p id="errorModalMessage"></p>
            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" data-bs-dismiss="modal"
                        onclick="closeErrorModal()">{% trans 'Close' %}</button>
            </div>
        </div>
    </div>
</div>
